package com.coherentlogic.coherent.data.model.core.builders;

import java.net.URI;

import javax.ws.rs.core.UriBuilder;

import org.springframework.web.client.RestTemplate;

/**
 * 
 *
 *
 * @author <a href="mailto:support@coherentlogic.com">Support</a>
 *
 * @todo Refactor this class as some of the logic needs to be shared between
 *  this class and the fred client.
 */
public abstract class AbstractQueryBuilder implements HttpMethodsSpecification {

    private final RestTemplate restTemplate;

    private final UriBuilder uriBuilder;

    static UriBuilder newUriBuilder (String uri) {
        UriBuilder builder;

        builder = UriBuilder.fromPath(uri);

//        try {
            builder = UriBuilder.fromPath(uri);
//            builder = new UriBuilder(uri);
//        } catch (URISyntaxException uriSyntaxException) {
//            throw new InvalidURIException (
//                "The uri '" + uri + "' is invalid.", uriSyntaxException);
//        }
        return builder;
    }

    protected AbstractQueryBuilder (
        RestTemplate restTemplate,
        String uri
    ) {
        this (restTemplate, newUriBuilder (uri));
    }

    protected AbstractQueryBuilder (
        RestTemplate restTemplate,
        UriBuilder uriBuilder
    ) {
        this.restTemplate = restTemplate;
        this.uriBuilder = uriBuilder;
    }

    /**
     * Method adds a name-value pair to the internal list of name-value pairs.
     *
     * @param name The name of the parameter.
     * @param value The parameter value.
     *
     * @throws IllegaStateException If either the name or value is null.
     */
    protected void addParameter (String name, String value) {

        // The uriBuilder will throw an exception if the name is null. We add
        // an additional check so that an exception is thrown if the value is
        // null. The reason for this is that the parameter should not be added
        // unless there's an appropriate value.
        if (name == null || value == null)
            throw new IllegalArgumentException("The name and value must " +
                "both be set to non-null values (name: " + name + ", value: " +
                value + ").");

        uriBuilder.queryParam(name, value);//addParameter(name, value);
    }

    /**
     * Method extends the {@link #uriBuilder}'s path with the path value -- ie.
     * 
     * http://www.foo.bar/ becomes http://www.foo.bar/baz/.
     * 
     * @param path The additional path value -- in the example above, 'baz'.
     */
    protected AbstractQueryBuilder extendPathWith (String path) {
        uriBuilder.path(path);

        return this;
    }

    /**
     * Method returns the escaped URI that is actually send to the World Bank
     * web service when the execute method has been called.
     *
     * This method can be useful when debugging, just print the escaped URI and
     * paste it into the address bar in your browser and see what's returned.
     *
     * @return A sting representation of the escaped URI.
     *
     * @todo This method needs to go in a parent class.
     */
    public String getEscapedURI () {

        URI uri;

//        try {
            uri = uriBuilder.build();
//        } catch (URISyntaxException uriSyntaxException) {
//            throw new InvalidURIException(
//                "The uri is invalid.", uriSyntaxException);
//        }

        String escapedURI = uri.toASCIIString();

        return escapedURI;
    }

    /**
     * Getter method for the RestTemplate.
     */
    protected RestTemplate getRestTemplate() {
        return restTemplate;
    }

    /**
     * Getter method for the RestTemplate.
     */
    protected UriBuilder getUriBuilder() {
        return uriBuilder;
    }

    /**
     *
     * @param type
     * @return
     *
     * @todo This method needs to go in a parent class.
     */
    public <T> T doGet (Class<T> type) {

        String escapedURI = getEscapedURI();

        T result = restTemplate.getForObject(escapedURI, type);

        return result;
    }
}
